package com.conflux.web.controller.contract.service.impl;


import com.conflux.common.utils.DateUtils;
import com.conflux.web.controller.contract.domain.ContractConfig;
import com.conflux.web.controller.contract.mapper.ContractConfigMapper;
import com.conflux.web.controller.contract.service.IContractConfigService;
import com.conflux.web.controller.util.AESUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 合约私钥配置Service业务层处理
 *
 * @author conflux
 * @date 2022-05-31
 */
@Service
public class ContractConfigServiceImpl implements IContractConfigService {
    @Autowired
    private ContractConfigMapper contractConfigMapper;

    /**
     * 查询合约私钥配置
     *
     * @param id 合约私钥配置主键
     * @return 合约私钥配置
     */
    @Override
    public ContractConfig selectContractConfigById(Long id) {
        ContractConfig contractConfig = contractConfigMapper.selectContractConfigById(id);
        try {
            contractConfig.setPrivateKey(AESUtil.decrypt(contractConfig.getPrivateKey()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return contractConfig;
    }

    /**
     * 查询合约私钥配置列表
     *
     * @param contractConfig 合约私钥配置
     * @return 合约私钥配置
     */
    @Override
    public List<ContractConfig> selectContractConfigList(ContractConfig contractConfig) {
        return contractConfigMapper.selectContractConfigList(contractConfig);
    }

    /**
     * 新增合约私钥配置
     *
     * @param contractConfig 合约私钥配置
     * @return 结果
     */
    @Override
    public int insertContractConfig(ContractConfig contractConfig) {
        contractConfig.setCreateTime(DateUtils.getNowDate());
        try {
            contractConfig.setPrivateKey(AESUtil.encrypt(contractConfig.getPrivateKey()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return contractConfigMapper.insertContractConfig(contractConfig);
    }

    /**
     * 修改合约私钥配置
     *
     * @param contractConfig 合约私钥配置
     * @return 结果
     */
    @Override
    public int updateContractConfig(ContractConfig contractConfig) {
        contractConfig.setUpdateTime(DateUtils.getNowDate());
        try {
            contractConfig.setPrivateKey(AESUtil.encrypt(contractConfig.getPrivateKey()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return contractConfigMapper.updateContractConfig(contractConfig);
    }

    /**
     * 批量删除合约私钥配置
     *
     * @param ids 需要删除的合约私钥配置主键
     * @return 结果
     */
    @Override
    public int deleteContractConfigByIds(Long[] ids) {
        return contractConfigMapper.deleteContractConfigByIds(ids);
    }

    /**
     * 删除合约私钥配置信息
     *
     * @param id 合约私钥配置主键
     * @return 结果
     */
    @Override
    public int deleteContractConfigById(Long id) {
        return contractConfigMapper.deleteContractConfigById(id);
    }

    @Override
    public ContractConfig selectContractConfigByUnitName(String unitName) {
        return contractConfigMapper.selectContractConfigByUnitName(unitName);
    }
}
