package com.conflux.web.controller.nft.service.impl;


import com.conflux.web.controller.collect.domain.CollectConfig;
import com.conflux.web.controller.collect.service.ICollectConfigService;
import com.conflux.web.controller.nft.domain.NftCollection;
import com.conflux.web.controller.nft.domain.dto.NftDTO;
import com.conflux.web.controller.nft.mapper.NftCollectionMapper;
import com.conflux.web.controller.nft.service.INftCollectionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.Date;
import java.util.List;

/**
 * nft上链Service业务层处理
 *
 * @author conflux
 * @date 2022-06-02
 */
@Service
@Slf4j
public class NftCollectionServiceImpl implements INftCollectionService {
    @Autowired
    private NftCollectionMapper nftCollectionMapper;
    @Autowired
    private ICollectConfigService collectConfigService;

    /**
     * 查询nft上链
     *
     * @param id nft上链主键
     * @return nft上链
     */
    @Override
    public NftCollection selectNftCollectionById(Long id) {
        return nftCollectionMapper.selectNftCollectionById(id);
    }

    /**
     * 查询nft上链列表
     *
     * @param nftCollection nft上链
     * @return nft上链
     */
    @Override
    public List<NftCollection> selectNftCollectionList(NftCollection nftCollection) {
        return nftCollectionMapper.selectNftCollectionList(nftCollection);
    }

    @Override
    public List<NftCollection> selectNftCollectionListByIds(String[] stringIds,Integer onChainStatus) {
        return nftCollectionMapper.selectNftCollectionListByIds(stringIds,onChainStatus);
    }

    @Override
    public int insertNftCollection(NftCollection nftCollection) {
        return nftCollectionMapper.insertNftCollection(nftCollection);
    }

    /**
     * 新增nft上链
     *
     * @param unitName,strings nft上链
     * @return 结果
     */
    @Override
    public int insertListNftCollection(String unitName, List<String> strings) {

        return nftCollectionMapper.insertListNftCollection(unitName, strings,new Date());
    }

    /**
     * 修改nft上链
     *
     * @param nftCollection nft上链
     * @return 结果
     */
    @Override
    public int updateNftCollection(NftCollection nftCollection) {
        return nftCollectionMapper.updateNftCollection(nftCollection);
    }

    /**
     * 批量删除nft上链
     *
     * @param ids 需要删除的nft上链主键
     * @return 结果
     */
    @Override
    public int deleteNftCollectionByIds(Long[] ids) {
        return nftCollectionMapper.deleteNftCollectionByIds(ids);
    }

    /**
     * 删除nft上链信息
     *
     * @param id nft上链主键
     * @return 结果
     */
    @Override
    public int deleteNftCollectionById(Long id) {
        return nftCollectionMapper.deleteNftCollectionById(id);
    }

    @Override
    public void updateCollect(NftDTO nftDTO) {
        BigInteger tokenId = nftDTO.getTokenId();
        String hexTokenId = tokenId.toString(16);
        if (hexTokenId.length() < 32) {
            int len = 32 - hexTokenId.length();
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < len; i++) {
                sb.append("0");
            }
            hexTokenId = sb.toString() + hexTokenId;
        }
        NftCollection nftCollection = nftCollectionMapper.selectNftCollectionByIdSting(hexTokenId);
        if (nftCollection == null) {
            log.info("[WjMyCollection][Not exist]{}", nftDTO);
            return;
        }
        if (nftCollection.getOnChainStatus() == 2) {
            log.info("[WjMyCollection][has mint]{}", nftDTO);
            return;
        }
        int row = nftCollectionMapper.updateByTokenId(hexTokenId,new Date());
        if (row < 1) {
            log.info("[WjMyCollection][updateCollect]{}", nftDTO);
            throw new RuntimeException("updateCollect error");
        }
    }
}
