package com.conflux.web.controller.nft.service.impl;


import com.conflux.common.utils.DateUtils;
import com.conflux.web.controller.nft.domain.NftLog;
import com.conflux.web.controller.nft.mapper.NftLogMapper;
import com.conflux.web.controller.nft.service.INftLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * nft日志Service业务层处理
 * 
 * @author conflux
 * @date 2022-06-02
 */
@Service
public class NftLogServiceImpl implements INftLogService
{
    @Autowired
    private NftLogMapper nftLogMapper;

    /**
     * 查询nft日志
     * 
     * @param id nft日志主键
     * @return nft日志
     */
    @Override
    public NftLog selectNftLogById(Long id)
    {
        return nftLogMapper.selectNftLogById(id);
    }

    /**
     * 查询nft日志列表
     * 
     * @param nftLog nft日志
     * @return nft日志
     */
    @Override
    public List<NftLog> selectNftLogList(NftLog nftLog)
    {
        return nftLogMapper.selectNftLogList(nftLog);
    }

    /**
     * 新增nft日志
     * 
     * @param nftLog nft日志
     * @return 结果
     */
    @Override
    public int insertNftLog(NftLog nftLog)
    {
        nftLog.setCreateTime(DateUtils.getNowDate());
        return nftLogMapper.insertNftLog(nftLog);
    }

    /**
     * 修改nft日志
     * 
     * @param nftLog nft日志
     * @return 结果
     */
    @Override
    public int updateNftLog(NftLog nftLog)
    {
        return nftLogMapper.updateNftLog(nftLog);
    }

    /**
     * 批量删除nft日志
     * 
     * @param ids 需要删除的nft日志主键
     * @return 结果
     */
    @Override
    public int deleteNftLogByIds(Long[] ids)
    {
        return nftLogMapper.deleteNftLogByIds(ids);
    }

    /**
     * 删除nft日志信息
     * 
     * @param id nft日志主键
     * @return 结果
     */
    @Override
    public int deleteNftLogById(Long id)
    {
        return nftLogMapper.deleteNftLogById(id);
    }

    @Override
    public int selectSum(String unitName) {
        String sum = nftLogMapper.selectSum(unitName);
        if (null!=sum){
            return Integer.valueOf(sum);
        }
        return 0;
    }
}
