package com.conflux.web.controller.privateKey.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.conflux.common.annotation.Log;
import com.conflux.common.core.controller.BaseController;
import com.conflux.common.core.domain.AjaxResult;
import com.conflux.common.enums.BusinessType;
import com.conflux.web.controller.privateKey.domain.PrivateKey;
import com.conflux.web.controller.privateKey.service.IPrivateKeyService;
import com.conflux.common.utils.poi.ExcelUtil;
import com.conflux.common.core.page.TableDataInfo;

/**
 * 私钥申请Controller
 * 
 * @author conflux
 * @date 2022-06-07
 */
@RestController
@RequestMapping("/conflux/key")
public class PrivateKeyController extends BaseController
{
    @Autowired
    private IPrivateKeyService privateKeyService;

    /**
     * 查询私钥申请列表
     */
    @PreAuthorize("@ss.hasPermi('system:key:list')")
    @GetMapping("/list")
    public TableDataInfo list(PrivateKey privateKey)
    {
        startPage();
        List<PrivateKey> list = privateKeyService.selectPrivateKeyList(privateKey);
        return getDataTable(list);
    }

    /**
     * 导出私钥申请列表
     */
    @PreAuthorize("@ss.hasPermi('system:key:export')")
    @Log(title = "私钥申请", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PrivateKey privateKey)
    {
        List<PrivateKey> list = privateKeyService.selectPrivateKeyList(privateKey);
        ExcelUtil<PrivateKey> util = new ExcelUtil<PrivateKey>(PrivateKey.class);
        util.exportExcel(response, list, "私钥申请数据");
    }

    /**
     * 获取私钥申请详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:key:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(privateKeyService.selectPrivateKeyById(id));
    }

    /**
     * 新增私钥申请
     */
    @PreAuthorize("@ss.hasPermi('system:key:add')")
    @Log(title = "私钥申请", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PrivateKey privateKey)
    {
        return toAjax(privateKeyService.insertPrivateKey(privateKey));
    }

    /**
     * 修改私钥申请
     */
    @PreAuthorize("@ss.hasPermi('system:key:edit')")
    @Log(title = "私钥申请", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PrivateKey privateKey)
    {
        return toAjax(privateKeyService.updatePrivateKey(privateKey));
    }

    /**
     * 删除私钥申请
     */
    @PreAuthorize("@ss.hasPermi('system:key:remove')")
    @Log(title = "私钥申请", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(privateKeyService.deletePrivateKeyByIds(ids));
    }


    @PreAuthorize("@ss.hasPermi('system:key:edit')")
    @Log(title = "私钥申请", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody PrivateKey privateKey)
    {
        return toAjax(privateKeyService.updatePrivateKey(privateKey));
    }
}
