package com.conflux.web.controller.util;

import com.conflux.common.utils.sign.Md5Utils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.security.SecureRandom;

/**
 * AES加密解密工具类（提供对称加密）
 */
@Slf4j
public final class AESUtil {
    // 加密密钥
    private static String KEY = "zyxXV3mYqgEuXk02IJQnLwQ7cb2LkOSv";
    // 默认字符集
    private static final String DEFAULT_CHARSET = "UTF-8";
    // 加密方式
    private static final String KEY_AES = "AES";
    // 加密模式
    private static final int EN_CODE = Cipher.ENCRYPT_MODE;
    // 解密模式
    private static final int DE_CODE = Cipher.DECRYPT_MODE;

    /**
     * k
     * 加密
     *
     * @param data 需要加密的数据
     * @return 返回加密后的结果
     */
    public static String encrypt(String data) throws Exception {
        return doAES(data, KEY, EN_CODE);
    }

    /**
     * 解密
     *
     * @param data 需要解密的数据
     * @return 返回解密后的结果
     */
    public static String decrypt(String data) throws Exception {
        return doAES(data, KEY, DE_CODE);
    }

    /**
     * 加密
     *
     * @param data 需要加密的数据
     * @param key  需要加密的密钥
     * @return 返回解密后的结果
     */
    public static String encrypt(String data, String key) throws Exception {
        return doAES(data, key, EN_CODE);
    }

    /**
     * 解密
     *
     * @param data 需要解密的数据
     * @param key  需要解密的密钥
     * @return 返回解密后的结果
     */
    public static String decrypt(String data, String key) throws Exception {
        return doAES(data, key, DE_CODE);
    }

    /**
     * 加密或解密方法
     *
     * @param data  需要加密或解密的内容
     * @param key   加密或解密的密钥
     * @param model 处理模式
     * @return 返回处理结果
     */
    private static String doAES(String data, String key, final int model) throws Exception {
        try {
            data = data == null ? "" : data;
            key = key == null ? "" : key;

            if ("".equalsIgnoreCase(data) || "".equalsIgnoreCase(key)) {
                return null;
            }

            // 构造密钥生成器，指定为AES方式，不区分大小写
            KeyGenerator kgen = KeyGenerator.getInstance(KEY_AES);

            // 根据EcnodeRules规则初始化密钥生成器
            // 根据传入的字节数组，生成一个128位的随机源
            SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
            random.setSeed(key.getBytes("UTF-8"));

//            kgen.init(128, new SecureRandom(key.getBytes(DEFAULT_CHARSET)));

            kgen.init(128, random);

            // 产生原始对称密钥
            SecretKey secretKey = kgen.generateKey();

            // 获得原始对称密钥的字节数组
            byte[] enCodeFormat = secretKey.getEncoded();

            // 根据字节数组生成AES密钥
            SecretKeySpec keySpec = new SecretKeySpec(enCodeFormat, KEY_AES);

            // 根据指定的AES算法创建密码器
            Cipher cipher = Cipher.getInstance(KEY_AES);

            // 初始化密码器，第一个参数为加密(Encrypt_mode)或者解密解密(Decrypt_mode)操作，第二个参数为使用的KEY
            cipher.init(model, keySpec);

            byte[] content; // 用于存储加密内容
            byte[] result;
            switch (model) {
                case EN_CODE:
                    content = data.getBytes(DEFAULT_CHARSET);
                    result = cipher.doFinal(content);

                    // 将二进制转换成16进制
                    return parseByte2HexStr(result);
                case DE_CODE:
                    content = parseHexStr2Byte(data);
                    result = cipher.doFinal(content);

                    // 将二进制转为字符串
                    return new String(result, DEFAULT_CHARSET);
                default:
                    throw new RuntimeException("The Encrypt Model Is Wrong!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("====error" + e.getMessage());
            return null;
        }
    }

    /**
     * 将二进制转换成16进制
     *
     * @param buf 二进制数据
     * @return 返回16进制结果
     */
    public static String parseByte2HexStr(byte[] buf) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < buf.length; i++) {
            String hex = Integer.toHexString(buf[i] & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            sb.append(hex.toUpperCase());
        }
        return sb.toString();
    }

    /**
     * 将16进制转换为二进制
     *
     * @param hexStr 16进制字符串
     * @return 返回二进制数据
     */
    public static byte[] parseHexStr2Byte(String hexStr) {
        if (hexStr.length() < 1) {
            return null;
        }
        byte[] result = new byte[hexStr.length() / 2];
        for (int i = 0; i < hexStr.length() / 2; i++) {
            int high = Integer.parseInt(hexStr.substring(i * 2, i * 2 + 1), 16);
            int low = Integer.parseInt(hexStr.substring(i * 2 + 1, i * 2 + 2), 16);
            result[i] = (byte) (high * 16 + low);
        }
        return result;
    }
    // 加密
    public static String Encrypt(String sSrc, String sKey) throws Exception {
        if (sKey == null) {
            System.out.print("Key为空null");
            return null;
        }
        byte[] raw = sKey.getBytes("utf-8");
        SecretKeySpec keySpec = new SecretKeySpec(raw, "AES");
        Cipher cipher = Cipher.getInstance("AES/ECB/PKCS5Padding");//"算法/模式/补码方式"cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
        cipher.init(Cipher.ENCRYPT_MODE, keySpec);
        byte[] encrypted = cipher.doFinal(sSrc.getBytes("utf-8"));
        return new String(Base64.encodeBase64(encrypted));//此处使用BASE64做转码功能，同时能起到2次加密的作用。
    }


    // 解密
    public static String Decrypt(String sSrc, String sKey) throws Exception {
        try {
            // 判断Key是否正确
            if (sKey == null) {
                System.out.print("Key为空null");
                return null;
            }
            byte[] raw = sKey.getBytes("utf-8");
            SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
            Cipher cipher =Cipher.getInstance("AES/ECB/PKCS5Padding");
            cipher.init(Cipher.DECRYPT_MODE, skeySpec);
            byte[] encrypted1 = Base64.decodeBase64(sSrc.getBytes());//先用base64解密
            try {
                byte[] original = cipher.doFinal(encrypted1);
                String originalString = new String(original, "utf-8");
                return originalString;
            } catch (Exception e) {
                return null;
            }
        } catch (Exception ex) {
            System.out.println(ex.toString());
            return null;
        }
    }

    public static void main(String[] args) {
        try {
            //AES
            String encrypt = Encrypt("56241DA2A04C2D8BAEDFF600976B5FFE4BF4910A52DC139D9F77521FFD1D28CB220E079108CCEE934D5654D2901DF44AFAE3B80664AD25E86ACD750805C35507200240EA65ED1190E15FA76FBB853D56","CONFLUX@12345678");
           System.err.println(encrypt);
            //md5
//            String hash = Md5Utils.md5Salt("d2f95f5f15389bd508765957f8543bc48b8e28cdfcde059a41421c2e17336f10fa475bba8c50963d4a28f697b559fa8d");
//            System.err.println(hash);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
