package com.conflux.web.controller.util;

import org.apache.commons.lang3.StringUtils;
import org.web3j.abi.EventEncoder;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.*;
import org.web3j.abi.datatypes.generated.Bytes32;
import org.web3j.abi.datatypes.generated.Uint256;
import org.web3j.abi.datatypes.generated.Uint32;
import org.web3j.utils.Numeric;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * @author wangban
 */
public class CfxUtils {

    public static final String ADDRESS = "address";
    public static final String UINT256 = "uint256";
    public static final String UINT32 = "uint32";
    public static final String Utf8String = "string";
    public static final String BOOL = "bool";
    public static final String BYTES = "bytes";
    public static final String BYTES32 = "bytes32";
    public static final String ARRAY = "array";
    public static final String UINT256_ARRAY = "uint256Array";
    public static final String ADDRESS_ARRAY = "addressArray";

    public static String getEventKeccak(String method, List<String> inputParams) {
        Event event = new Event(method, getTypes(inputParams));
        return EventEncoder.encode(event);
    }

    public static String getUtf8String(String data) {
        return new String(Numeric.hexStringToByteArray(data)).trim();
    }

    public static List<String> getEventParams(String data) {
        if (data.startsWith("0x")) {
            data = data.substring(2);
        }
        List<String> result = new ArrayList<>();
        result.add(data);
        int length = data.length() / 64;
        for (int i = 0; i < length; i++) {
            String param = data.substring(i * 64, (i + 1) * 64);
            BigInteger b = new BigInteger(param, 16);
            result.add(b.toString(16));
        }
//        System.out.println(result);
        return result;
    }

    public static String getAddress(String address) {
        if (address.length() > 42) {
            address = address.substring(26);
        }
        if (address.startsWith("0x")) {
            return address;
        }
        if (StringUtils.isEmpty(address) || "0".equals(address)) {
            return "0x0000000000000000000000000000000000000000";
        }
        return "0x" + address;
    }

    public static void main(String[] args) {
        System.out.println(getAddress("0x0000000000000000000000001cba0557e0b38e595588b0f8a3d7c89de992e4e3"));
    }

    public static BigInteger getChainId(BigInteger chainId){
        if(chainId.compareTo(BigInteger.ONE)==0){
            return new BigInteger("10001");
        }else {
            return new BigInteger("11029");
        }
    }

    public static BigInteger getNumber(String num) {
        if (num.contains("0x")) {
            num = num.substring(2);
        }
        return new BigInteger(num, 16);
    }

    public static Integer getIntNumber(String num) {
        if (num.contains("0x")) {
            num = num.substring(2);
        }
        return new BigInteger(num, 16).intValue();
    }


    private static List<TypeReference<?>> getTypes(List<String> inputParams) {
        List<TypeReference<?>> parameters = new ArrayList<>();
        for (int i = 0; i < inputParams.size(); i++) {
            String in = inputParams.get(i);
            switch (in) {
                case ADDRESS:
                    parameters.add(new TypeReference<Address>() {
                    });
                    break;
                case UINT256:
                    parameters.add(new TypeReference<Uint256>() {
                    });
                    break;
                case UINT32:
                    parameters.add(new TypeReference<Uint32>() {
                    });
                    break;
                case Utf8String:
                    parameters.add(new TypeReference<org.web3j.abi.datatypes.Utf8String>() {
                    });
                    break;
                case BOOL:
                    parameters.add(new TypeReference<Bool>() {
                    });
                    break;
                case BYTES:
                    parameters.add(new TypeReference<Bytes>() {
                    });
                    break;
                case BYTES32:
                    parameters.add(new TypeReference<Bytes32>() {
                    });
                    break;
                case ARRAY:
                    parameters.add(new TypeReference<Array>() {
                    });
                    break;
                case UINT256_ARRAY:
                    parameters.add(new TypeReference<DynamicArray<Uint256>>() {
                    });
                    break;
                case ADDRESS_ARRAY:
                    parameters.add(new TypeReference<DynamicArray<Address>>() {
                    });
                    break;
                default:
                    break;
            }
        }
        return parameters;
    }

}
