package com.conflux.web.controller.util;

import cn.hutool.core.date.DateUtil;
import com.conflux.web.controller.collect.domain.CollectConfig;
import com.conflux.web.controller.collect.service.ICollectConfigService;
import com.conflux.web.controller.nft.domain.EventParam;
import com.conflux.web.controller.nft.domain.vo.FilterMapVO;
import com.conflux.web.controller.nft.service.IHandlerStrategy;
import com.google.gson.Gson;
import conflux.web3j.Cfx;
import conflux.web3j.Request;
import conflux.web3j.request.Epoch;
import conflux.web3j.request.LogFilter;
import conflux.web3j.response.Log;
import conflux.web3j.types.Address;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author _007long
 * @Date 2022 06 07
 **/
@Slf4j
@Configuration
public class ScheduledTask {
    @Value("${conflux.contract}")
    private String contract;
    @Resource
    private ICollectConfigService collectConfigService;
    @Resource
    private IHandlerStrategy iHandlerStrategy;
    private Map<String,CollectConfig> map=new HashMap();
    //@Scheduled(fixedDelay = 5000L)
    public void executeMakeUp() {
        CollectConfig collect = collectConfigService.selectCollectConfigByStatus();
        if (collect.getOnPause()) {
            log.info("[dispatchHandler]{}", "暂停中");
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            return;
        }
        //=======检查数据=======
        List<EventParam> eventParams = new ArrayList<>();
        EventParam eventParam = new EventParam();
        eventParam.setChainId(collect.getChainId());
        eventParam.setContract(contract);//合约
        eventParam.setParam("address,address,uint256");
        eventParam.setMethod("Transfer");
        eventParam.setType(20000);
        eventParam.setRemark("Wu jing");
        eventParams.add(eventParam);
        if (CollectionUtils.isEmpty(eventParams)) {
            log.error("[dispatchHandler]{}", "没有需要监听的数据");
            return;
        }
        Gson gson = new Gson();
        //获取event，使用 webService
        //记录数据
        Cfx cfx = null;
        try {
            cfx = Cfx.create(collect.getNode(), 3, 1000);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
        FilterMapVO vo = getFilter(eventParams);
        CollectConfig collectConfig = map.get(contract);
        long from = 0l;
        if (null != collectConfig) {
            from = collectConfig.getEpochNumber();
        } else {
            from = collect.getEpochNumber();
        }
        // Invoke cfx method
        BigInteger epoch = cfx.getEpochNumber().sendAndGet();
        log.info("[executeMakeUp][Current epoch]:{}", epoch);
        long to = epoch.intValue() - 5;
        if (to - from > collect.getLimitCount()) {
            to = from + collect.getLimitCount();
        }
        vo.getLogFilter().setFromEpoch(Epoch.numberOf(from));
        //当前高度
        vo.getLogFilter().setToEpoch(Epoch.numberOf(to));
        Request<List<Log>, Log.Response> events = cfx.getLogs(vo.getLogFilter());
        try {
            events = cfx.getLogs(vo.getLogFilter());
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (events == null) {
            log.info("[executeMakeUp][请重启服务]");
            return;
        }
        List<Log> logList = events.sendAndGet();
        boolean isSuccess = false;
        if (CollectionUtils.isEmpty(logList)) {
            log.info("[executeMakeUp][此区间暂无日志]:from:{} to:{}", from, to);
            isSuccess = true;
        }
        for (Log l : logList) {
            //处理高度信息
            String evAddress = l.getTopics().get(0);
            String tokenAddress = l.getAddress().getAddress();
            if (tokenAddress.contains("TYPE")) {
                Address a = new Address(tokenAddress);
                tokenAddress = a.getAddress();
            }
            int type = vo.getMap().get(evAddress + "-" + tokenAddress.toLowerCase());
            if (type == 0) {
                log.info("[dispatchHandler][暂无符合条件的数据]");
                return;
            }
            boolean res;
            try {
                res = iHandlerStrategy.relayHandler(cfx, gson.toJson(l), type);
            } catch (Exception e) {
                e.printStackTrace();
                isSuccess = false;
                log.info("[dispatchHandler][数据错误未分配]{}", l.getData());
                break;
            }
            if (res) {
                log.info("[dispatchHandler][分配成功]{}", evAddress);
                isSuccess = true;
            } else {
                log.info("[dispatchHandler][数据错误未分配]{}", l.getData());
                isSuccess = false;
                break;
            }
        }
        //分析数据拉取成功
        CollectConfig t = new CollectConfig();
        t.setId(collect.getId());
        t.setUpdateTime(DateUtil.currentSeconds());
        if (isSuccess) {
            t.setEpochNumber(to);
            log.info("[dispatchHandler][更新高度]:{}", to);
            map.put(contract,t);
        }
    }

    /**
     * 每隔一分钟更新一次数据
     */
    //@Scheduled(fixedDelay = 60000L)
    public void flushDiskToMysql(){
        CollectConfig collectConfig = map.get(contract);
        if (null!=collectConfig){
            //写入数据库
            collectConfigService.updateCollectConfig(collectConfig);
            System.err.println(map.toString());
        }else {
            System.err.println("写入数据库失败！");
        }

    }
    private FilterMapVO getFilter(List<EventParam> eventParams) {
        // PubSub Subscribe to incoming events and process incoming events
        LogFilter filter = new LogFilter();
        //添加监听合约：
        List<Address> moList = eventParams.stream().map(
                eventParam -> new Address(eventParam.getContract())).collect(Collectors.toList());
        filter.setAddress(moList);
        //添加监听方法
        List<String> list = new ArrayList<>();
        //Transfer
        Map<String, Integer> map = new HashMap<>(eventParams.size());
        eventParams.forEach(eventParam -> {
            String evAddress = CfxUtils.getEventKeccak(eventParam.getMethod(), Arrays.asList(eventParam.getParam().split(",")));
//            log.info("token:{} method:{} kec:{}", eventParam.getContract(), eventParam.getMethod(), evAddress);
            map.put(evAddress + "-" + eventParam.getContract().toLowerCase(), eventParam.getType());
            list.add(evAddress);
        });
        List<List<String>> lists = new ArrayList<>();
        lists.add(list);
        filter.setTopics(lists);
        FilterMapVO vo = new FilterMapVO();
        vo.setLogFilter(filter);
        vo.setMap(map);
        return vo;
    }

}
